@@ -16,6 +16,8 @@
 #pragma once
 
 #include "../stdexec/execution.hpp"
+#include "../stdexec/__detail/__any_receiver_ref.hpp"
+#include "../stdexec/__detail/__transform_completion_signatures.hpp"
 
 #include "sequence_senders.hpp"
 
@@ -82,7 +84,8 @@ namespace exec {
     struct __query_vfun_fn<_EnvProvider, true> {
       template <class _Tag, class _Ret, class... _As>
         requires __callable<_Tag, env_of_t<const _EnvProvider&>, _As...>
-      constexpr _Ret (*operator()(_Tag (*)(_Ret (*)(_As...))) const noexcept)(void*, _As...) {
+      constexpr auto
+        operator()(_Tag (*)(_Ret (*)(_As...))) const noexcept -> _Ret (*)(void*, _As...) {
         return +[](void* __env_provider, _As... __as) -> _Ret {
           return _Tag{}(
             stdexec::get_env(*static_cast<const _EnvProvider*>(__env_provider)),
@@ -92,8 +95,8 @@ namespace exec {
 
       template <class _Tag, class _Ret, class... _As>
         requires __callable<_Tag, env_of_t<const _EnvProvider&>, _As...>
-      constexpr _Ret (*operator()(_Tag (*)(_Ret (*)(_As...) noexcept))
-                        const noexcept)(void*, _As...) noexcept {
+      constexpr auto operator()(_Tag (*)(_Ret (*)(_As...) noexcept)) const noexcept
+        -> _Ret (*)(void*, _As...) noexcept {
         return +[](void* __env_provider, _As... __as) noexcept -> _Ret {
           static_assert(__nothrow_callable<_Tag, const env_of_t<_EnvProvider>&, _As...>);
           return _Tag{}(
@@ -107,7 +110,8 @@ namespace exec {
     struct __query_vfun_fn<_Queryable, false> {
       template <class _Tag, class _Ret, class... _As>
         requires __callable<_Tag, const _Queryable&, _As...>
-      constexpr _Ret (*operator()(_Tag (*)(_Ret (*)(_As...))) const noexcept)(void*, _As...) {
+      constexpr auto
+        operator()(_Tag (*)(_Ret (*)(_As...))) const noexcept -> _Ret (*)(void*, _As...) {
         return +[](void* __queryable, _As... __as) -> _Ret {
           return _Tag{}(*static_cast<const _Queryable*>(__queryable), static_cast<_As&&>(__as)...);
         };
@@ -115,8 +119,8 @@ namespace exec {
 
       template <class _Tag, class _Ret, class... _As>
         requires __callable<_Tag, const _Queryable&, _As...>
-      constexpr _Ret (*operator()(_Tag (*)(_Ret (*)(_As...) noexcept))
-                        const noexcept)(void*, _As...) noexcept {
+      constexpr auto operator()(_Tag (*)(_Ret (*)(_As...) noexcept)) const noexcept
+        -> _Ret (*)(void*, _As...) noexcept {
         return +[](void* __env_provider, _As... __as) noexcept -> _Ret {
           static_assert(__nothrow_callable<_Tag, const _Queryable&, _As...>);
           return _Tag{}(
@@ -152,7 +156,8 @@ namespace exec {
     struct __storage_vfun_fn {
       template <class _Tag, class... _As>
         requires __callable<_Tag, __mtype<_Tp>, _Storage&, _As...>
-      constexpr void (*operator()(_Tag (*)(void (*)(_As...))) const noexcept)(void*, _As...) {
+      constexpr auto
+        operator()(_Tag (*)(void (*)(_As...))) const noexcept -> void (*)(void*, _As...) {
         return +[](void* __storage, _As... __as) -> void {
           return _Tag{}(
             __mtype<_Tp>{}, *static_cast<_Storage*>(__storage), static_cast<_As&&>(__as)...);
@@ -161,8 +166,8 @@ namespace exec {
 
       template <class _Tag, class... _As>
         requires __callable<_Tag, __mtype<_Tp>, _Storage&, _As...>
-      constexpr void (*operator()(_Tag (*)(void (*)(_As...) noexcept))
-                        const noexcept)(void*, _As...) noexcept {
+      constexpr auto operator()(_Tag (*)(void (*)(_As...) noexcept)) const noexcept
+        -> void (*)(void*, _As...) noexcept {
         return +[](void* __storage, _As... __as) noexcept -> void {
           static_assert(__nothrow_callable<_Tag, __mtype<_Tp>, _Storage&, _As...>);
           return _Tag{}(
@@ -185,9 +190,8 @@ namespace exec {
     struct __copy_construct_t {
       template <class _Storage, class _Tp>
         requires tag_invocable<__copy_construct_t, __mtype<_Tp>, _Storage&, const _Storage&>
-      void operator()(__mtype<_Tp>, _Storage& __self, const _Storage& __from) //
-        noexcept(
-          nothrow_tag_invocable<__copy_construct_t, __mtype<_Tp>, _Storage&, const _Storage&>) {
+      void operator()(__mtype<_Tp>, _Storage& __self, const _Storage& __from) noexcept(
+        nothrow_tag_invocable<__copy_construct_t, __mtype<_Tp>, _Storage&, const _Storage&>) {
         stdexec::tag_invoke(__copy_construct_t{}, __mtype<_Tp>{}, __self, __from);
       }
     };
@@ -247,7 +251,8 @@ namespace exec {
       class _Allocator,
       bool _Copyable = false,
       std::size_t _InlineSize = 3 * sizeof(void*),
-      std::size_t _Alignment = alignof(std::max_align_t)>
+      std::size_t _Alignment = alignof(std::max_align_t)
+    >
     struct __storage {
       class __t;
     };
@@ -256,7 +261,8 @@ namespace exec {
       class _Vtable,
       class _Allocator,
       std::size_t _InlineSize = 3 * sizeof(void*),
-      std::size_t _Alignment = alignof(std::max_align_t)>
+      std::size_t _Alignment = alignof(std::max_align_t)
+    >
     struct __immovable_storage {
       class __t : __immovable {
         static constexpr std::size_t __buffer_size = std::max(_InlineSize, sizeof(void*));
@@ -265,8 +271,8 @@ namespace exec {
         using __vtable_t = __storage_vtable<_Vtable, __with_delete>;
 
         template <class _Tp>
-        static constexpr bool __is_small =
-          sizeof(_Tp) <= __buffer_size && alignof(_Tp) <= __alignment;
+        static constexpr bool __is_small = sizeof(_Tp) <= __buffer_size
+                                        && alignof(_Tp) <= __alignment;
 
         template <class _Tp>
         static constexpr auto __get_vtable_of_type() noexcept -> const __vtable_t* {
@@ -324,7 +330,7 @@ namespace exec {
         template <class _Tp, class... _As>
         void __construct_small(_As&&... __args) {
           static_assert(sizeof(_Tp) <= __buffer_size && alignof(_Tp) <= __alignment);
-          _Tp* __pointer = static_cast<_Tp*>(static_cast<void*>(&__buffer_[0]));
+          _Tp* __pointer = reinterpret_cast<_Tp*>(&__buffer_[0]);
           using _Alloc = typename std::allocator_traits<_Allocator>::template rebind_alloc<_Tp>;
           _Alloc __alloc{__allocator_};
           std::allocator_traits<_Alloc>::construct(
@@ -337,12 +343,13 @@ namespace exec {
           using _Alloc = typename std::allocator_traits<_Allocator>::template rebind_alloc<_Tp>;
           _Alloc __alloc{__allocator_};
           _Tp* __pointer = std::allocator_traits<_Alloc>::allocate(__alloc, 1);
-          try {
+          STDEXEC_TRY {
             std::allocator_traits<_Alloc>::construct(
               __alloc, __pointer, static_cast<_As&&>(__args)...);
-          } catch (...) {
+          }
+          STDEXEC_CATCH_ALL {
             std::allocator_traits<_Alloc>::deallocate(__alloc, __pointer, 1);
-            throw;
+            STDEXEC_THROW();
           }
           __object_pointer_ = __pointer;
         }
@@ -364,7 +371,7 @@ namespace exec {
         const __vtable_t* __vtable_{__default_storage_vtable(static_cast<__vtable_t*>(nullptr))};
         void* __object_pointer_{nullptr};
         alignas(__alignment) std::byte __buffer_[__buffer_size]{};
-        STDEXEC_ATTRIBUTE((no_unique_address)) _Allocator __allocator_{};
+        STDEXEC_ATTRIBUTE(no_unique_address) _Allocator __allocator_ { };
       };
     };
 
@@ -373,7 +380,8 @@ namespace exec {
       class _Allocator,
       bool _Copyable,
       std::size_t _InlineSize,
-      std::size_t _Alignment>
+      std::size_t _Alignment
+    >
     class __storage<_Vtable, _Allocator, _Copyable, _InlineSize, _Alignment>::__t
       : __if_c<_Copyable, __, __move_only> {
       static_assert(
@@ -392,12 +400,20 @@ namespace exec {
       using __vtable_t = __if_c<
         _Copyable,
         __storage_vtable<_Vtable, __with_delete, __with_move, __with_copy>,
-        __storage_vtable<_Vtable, __with_delete, __with_move>>;
+        __storage_vtable<_Vtable, __with_delete, __with_move>
+      >;
 
       template <class _Tp>
       static constexpr auto __get_vtable_of_type() noexcept -> const __vtable_t* {
         if constexpr (_Copyable) {
-          return &__storage_vtbl<__t, __decay_t<_Tp>, _Vtable, __with_delete, __with_move, __with_copy>;
+          return &__storage_vtbl<
+            __t,
+            __decay_t<_Tp>,
+            _Vtable,
+            __with_delete,
+            __with_move,
+            __with_copy
+          >;
         } else {
           return &__storage_vtbl<__t, __decay_t<_Tp>, _Vtable, __with_delete, __with_move>;
         }
@@ -433,11 +449,13 @@ namespace exec {
 
       __t(const __t& __other)
         requires(_Copyable)
-      {
+        : __vtable_(__other.__vtable_) {
         (*__other.__vtable_)(__copy_construct, this, __other);
       }
 
-      auto operator=(const __t& __other) -> __t& requires(_Copyable) {
+      auto operator=(const __t& __other) -> __t&
+        requires(_Copyable)
+      {
         if (&__other != this) {
           __t tmp(__other);
           *this = std::move(tmp);
@@ -445,7 +463,8 @@ namespace exec {
         return *this;
       }
 
-      __t(__t&& __other) noexcept {
+      __t(__t&& __other) noexcept
+        : __vtable_(__other.__vtable_) {
         (*__other.__vtable_)(__move_construct, this, static_cast<__t&&>(__other));
       }
 
@@ -478,7 +497,7 @@ namespace exec {
       template <class _Tp, class... _As>
       void __construct_small(_As&&... __args) {
         static_assert(sizeof(_Tp) <= __buffer_size && alignof(_Tp) <= __alignment);
-        _Tp* __pointer = static_cast<_Tp*>(static_cast<void*>(&__buffer_[0]));
+        _Tp* __pointer = reinterpret_cast<_Tp*>(&__buffer_[0]);
         using _Alloc = typename std::allocator_traits<_Allocator>::template rebind_alloc<_Tp>;
         _Alloc __alloc{__allocator_};
         std::allocator_traits<_Alloc>::construct(__alloc, __pointer, static_cast<_As&&>(__args)...);
@@ -490,12 +509,13 @@ namespace exec {
         using _Alloc = typename std::allocator_traits<_Allocator>::template rebind_alloc<_Tp>;
         _Alloc __alloc{__allocator_};
         _Tp* __pointer = std::allocator_traits<_Alloc>::allocate(__alloc, 1);
-        try {
+        STDEXEC_TRY {
           std::allocator_traits<_Alloc>::construct(
             __alloc, __pointer, static_cast<_As&&>(__args)...);
-        } catch (...) {
+        }
+        STDEXEC_CATCH_ALL {
           std::allocator_traits<_Alloc>::deallocate(__alloc, __pointer, 1);
-          throw;
+          STDEXEC_THROW();
         }
         __object_pointer_ = __pointer;
       }
@@ -536,7 +556,8 @@ namespace exec {
 
       template <class _Tp>
         requires _Copyable
-      STDEXEC_MEMFN_DECL(void __copy_construct)(this __mtype<_Tp>, __t& __self, const __t& __other) {
+      STDEXEC_MEMFN_DECL(
+        void __copy_construct)(this __mtype<_Tp>, __t& __self, const __t& __other) {
         if (!__other.__object_pointer_) {
           return;
         }
@@ -552,7 +573,7 @@ namespace exec {
       const __vtable_t* __vtable_{__default_storage_vtable(static_cast<__vtable_t*>(nullptr))};
       void* __object_pointer_{nullptr};
       alignas(__alignment) std::byte __buffer_[__buffer_size]{};
-      STDEXEC_ATTRIBUTE((no_unique_address)) _Allocator __allocator_{};
+      STDEXEC_ATTRIBUTE(no_unique_address) _Allocator __allocator_ { };
     };
 
     struct __empty_vtable {
@@ -569,7 +590,8 @@ namespace exec {
       class _VTable = __empty_vtable,
       class _Allocator = std::allocator<std::byte>,
       std::size_t _InlineSize = 3 * sizeof(void*),
-      std::size_t _Alignment = alignof(std::max_align_t)>
+      std::size_t _Alignment = alignof(std::max_align_t)
+    >
     using __immovable_storage_t =
       __t<__immovable_storage<_VTable, _Allocator, _InlineSize, _Alignment>>;
 
@@ -579,7 +601,8 @@ namespace exec {
     template <
       class _VTable,
       std::size_t _InlineSize = 3 * sizeof(void*),
-      class _Allocator = std::allocator<std::byte>>
+      class _Allocator = std::allocator<std::byte>
+    >
     using __copyable_storage_t = __t<__storage<_VTable, _Allocator, true, _InlineSize>>;
 
     template <class _Tag, class... _As>
@@ -596,7 +619,6 @@ namespace exec {
     template <class _Query>
     concept __is_not_stop_token_query = !__is_stop_token_query<_Query>;
 
-
     template <class _Query>
     using __is_not_stop_token_query_v = __mbool<__is_not_stop_token_query<_Query>>;
 
@@ -611,22 +633,33 @@ namespace exec {
 
       template <class... _Sigs, class... _Queries>
       struct __vtable<completion_signatures<_Sigs...>, _Queries...> {
-        class __t
-          : public __any_::__rcvr_vfun<_Sigs>...
-          , public __query_vfun<_Queries>... {
-         public:
+        struct __t
+          : __overload<__any_::__rcvr_vfun<_Sigs>...>
+          , __query_vfun<_Queries>... {
           using __query_vfun<_Queries>::operator()...;
-          using __any_::__rcvr_vfun<_Sigs>::operator()...;
+
+          template <class _Tag, class... _As>
+            requires __one_of<_Tag(_As...), _Sigs...>
+                  || __callable<__overload<__any_::__rcvr_vfun<_Sigs>...>, void*, _Tag, _As...>
+          void operator()(void* __rcvr, _Tag, _As&&... __as) const noexcept {
+            if constexpr (__one_of<_Tag(_As...), _Sigs...>) {
+              const __any_::__rcvr_vfun<_Tag(_As...)>& __vfun = *this;
+              __vfun(__rcvr, _Tag(), static_cast<_As&&>(__as)...);
+            } else {
+              const __overload<__any_::__rcvr_vfun<_Sigs>...>& __vfun = *this;
+              __vfun(__rcvr, _Tag(), static_cast<_As&&>(__as)...);
+            }
+          }
 
          private:
           template <class _Rcvr>
             requires receiver_of<_Rcvr, completion_signatures<_Sigs...>>
                   && (__callable<__query_vfun_fn<_Rcvr>, _Queries> && ...)
-          STDEXEC_MEMFN_DECL(
-            auto __create_vtable)(this __mtype<__t>, __mtype<_Rcvr>) noexcept -> const __t* {
+          STDEXEC_MEMFN_DECL(auto __create_vtable)(this __mtype<__t>, __mtype<_Rcvr>) noexcept
+            -> const __t* {
             static const __t __vtable_{
-              {__any_::__rcvr_vfun_fn(
-                static_cast<_Rcvr*>(nullptr), static_cast<_Sigs*>(nullptr))}...,
+              {{__any_::__rcvr_vfun_fn(
+                static_cast<_Rcvr*>(nullptr), static_cast<_Sigs*>(nullptr))}...},
               {__query_vfun_fn<_Rcvr>{}(static_cast<_Queries>(nullptr))}...};
             return &__vtable_;
           }
@@ -638,7 +671,6 @@ namespace exec {
       struct __ref<completion_signatures<_Sigs...>, _Queries...> {
 #if !STDEXEC_MSVC()
         // MSVCBUG https://developercommunity.visualstudio.com/t/Private-member-inaccessible-when-used-in/10448363
-
        private:
 #endif
         using __vtable_t = stdexec::__t<__vtable<completion_signatures<_Sigs...>, _Queries...>>;
@@ -650,12 +682,13 @@ namespace exec {
 
           template <class _Tag, class... _As>
             requires __callable<const __vtable_t&, _Tag, void*, _As...>
-          auto query(_Tag, _As&&... __as) const //
+          auto query(_Tag, _As&&... __as) const
             noexcept(__nothrow_callable<const __vtable_t&, _Tag, void*, _As...>)
               -> __call_result_t<const __vtable_t&, _Tag, void*, _As...> {
             return (*__vtable_)(_Tag{}, __rcvr_, static_cast<_As&&>(__as)...);
           }
 
+          [[nodiscard]]
           auto query(get_stop_token_t) const noexcept -> inplace_stop_token {
             return __token_;
           }
@@ -708,8 +741,8 @@ namespace exec {
       auto __test_never_stop_token(_Tag (*)(_Ret (*)(_As...))) -> __mbool<false>;
 
       template <class _Query>
-      using __is_never_stop_token_query =
-        decltype(__test_never_stop_token(static_cast<_Query>(nullptr)));
+      using __is_never_stop_token_query = decltype(__test_never_stop_token(
+        static_cast<_Query>(nullptr)));
 
       template <class... _Sigs, class... _Queries>
         requires(__is_stop_token_query<_Queries> || ...)
@@ -722,7 +755,8 @@ namespace exec {
         using _FilteredQueries =
           __minvoke<__mremove_if<__q<__is_never_stop_token_query>>, _Queries...>;
         using __vtable_t = stdexec::__t<
-          __mapply<__mbind_front_q<__vtable, completion_signatures<_Sigs...>>, _FilteredQueries>>;
+          __mapply<__mbind_front_q<__vtable, completion_signatures<_Sigs...>>, _FilteredQueries>
+        >;
 
         struct __env_t {
           const __vtable_t* __vtable_;
@@ -730,7 +764,7 @@ namespace exec {
 
           template <class _Tag, class... _As>
             requires __callable<const __vtable_t&, _Tag, void*, _As...>
-          auto query(_Tag, _As&&... __as) const //
+          auto query(_Tag, _As&&... __as) const
             noexcept(__nothrow_callable<const __vtable_t&, _Tag, void*, _As...>)
               -> __call_result_t<const __vtable_t&, _Tag, void*, _As...> {
             return (*__vtable_)(_Tag{}, __rcvr_, static_cast<_As&&>(__as)...);
@@ -750,23 +784,35 @@ namespace exec {
 
         template <class... _As>
           requires __one_of<set_value_t(_As...), _Sigs...>
+                || __callable<__overload<__any_::__rcvr_vfun<_Sigs>...>, void*, set_value_t, _As...>
         void set_value(_As&&... __as) noexcept {
-          const __any_::__rcvr_vfun<set_value_t(_As...)>* __vfun = __env_.__vtable_;
-          (*__vfun->__complete_)(__env_.__rcvr_, static_cast<_As&&>(__as)...);
+          if constexpr (__one_of<set_value_t(_As...), _Sigs...>) {
+            const __any_::__rcvr_vfun<set_value_t(_As...)>& __vfun = *__env_.__vtable_;
+            __vfun(__env_.__rcvr_, set_value_t(), static_cast<_As&&>(__as)...);
+          } else {
+            const __overload<__any_::__rcvr_vfun<_Sigs>...>& __vfun = *__env_.__vtable_;
+            __vfun(__env_.__rcvr_, set_value_t(), static_cast<_As&&>(__as)...);
+          }
         }
 
         template <class _Error>
           requires __one_of<set_error_t(_Error), _Sigs...>
+                || __callable<__overload<__any_::__rcvr_vfun<_Sigs>...>, void*, set_error_t, _Error>
         void set_error(_Error&& __err) noexcept {
-          const __any_::__rcvr_vfun<set_error_t(_Error)>* __vfun = __env_.__vtable_;
-          (*__vfun->__complete_)(__env_.__rcvr_, static_cast<_Error&&>(__err));
+          if constexpr (__one_of<set_error_t(_Error), _Sigs...>) {
+            const __any_::__rcvr_vfun<set_error_t(_Error)>& __vfun = *__env_.__vtable_;
+            __vfun(__env_.__rcvr_, set_error_t(), static_cast<_Error&&>(__err));
+          } else {
+            const __overload<__any_::__rcvr_vfun<_Sigs>...>& __vfun = *__env_.__vtable_;
+            __vfun(__env_.__rcvr_, set_error_t(), static_cast<_Error&&>(__err));
+          }
         }
 
         void set_stopped() noexcept
           requires __one_of<set_stopped_t(), _Sigs...>
         {
-          const __any_::__rcvr_vfun<set_stopped_t()>* __vfun = __env_.__vtable_;
-          (*__vfun->__complete_)(__env_.__rcvr_);
+          const __any_::__rcvr_vfun<set_stopped_t()>& __vfun = *__env_.__vtable_;
+          __vfun(__env_.__rcvr_, set_stopped_t());
         }
 
         auto get_env() const noexcept -> const __env_t& {
@@ -798,7 +844,7 @@ namespace exec {
       __immovable_storage_t<__operation_vtable, std::allocator<std::byte>, 6 * sizeof(void*)>;
 
     template <class _Sigs, class _Queries>
-    using __receiver_ref = __mapply<__mbind_front<__q<__rec::__ref>, _Sigs>, _Queries>;
+    using __receiver_ref = __mapply<__mbind_front_q<__rec::__ref, _Sigs>, _Queries>;
 
     struct __on_stop_t {
       stdexec::inplace_stop_source& __source_;
@@ -810,15 +856,16 @@ namespace exec {
 
     template <class _Receiver>
     struct __operation_base {
-      STDEXEC_ATTRIBUTE((no_unique_address)) _Receiver __rcvr_;
+      STDEXEC_ATTRIBUTE(no_unique_address) _Receiver __rcvr_;
       stdexec::inplace_stop_source __stop_source_{};
       using __stop_callback = typename stdexec::stop_token_of_t<
-        stdexec::env_of_t<_Receiver>>::template callback_type<__on_stop_t>;
+        stdexec::env_of_t<_Receiver>
+      >::template callback_type<__on_stop_t>;
       std::optional<__stop_callback> __on_stop_{};
     };
 
     template <class _Env>
-    using __env_t = __env::__join_t<prop<get_stop_token_t, inplace_stop_token>, _Env>;
+    using __env_t = __join_env_t<prop<get_stop_token_t, inplace_stop_token>, _Env>;
 
     template <class _ReceiverId>
     struct __stoppable_receiver {
@@ -828,11 +875,12 @@ namespace exec {
         using receiver_concept = stdexec::receiver_t;
         __operation_base<_Receiver>* __op_;
 
-        template <same_as<__t> _Self, class _Item>
+        template <class _Item>
           requires __callable<set_next_t, _Receiver&, _Item>
-        STDEXEC_MEMFN_DECL(auto set_next)(this _Self& __self, _Item&& __item) noexcept
+        [[nodiscard]]
+        auto set_next(_Item&& __item) & noexcept(__nothrow_callable<set_next_t, _Receiver&, _Item>)
           -> __call_result_t<set_next_t, _Receiver&, _Item> {
-          return exec::set_next(__self.__op_->__rcvr_, static_cast<_Item&&>(__item));
+          return exec::set_next(__op_->__rcvr_, static_cast<_Item&&>(__item));
         }
 
         template <class... _Args>
@@ -847,7 +895,8 @@ namespace exec {
           requires __callable<set_error_t, _Receiver, _Error>
         void set_error(_Error&& __err) noexcept {
           __op_->__on_stop_.reset();
-          stdexec::set_error(static_cast<_Receiver&&>(__op_->__rcvr_), static_cast<_Error&&>(__err));
+          stdexec::set_error(
+            static_cast<_Receiver&&>(__op_->__rcvr_), static_cast<_Error&&>(__err));
         }
 
         void set_stopped() noexcept
@@ -917,7 +966,7 @@ namespace exec {
         }
 
        private:
-        STDEXEC_ATTRIBUTE((no_unique_address)) _Receiver __rec_;
+        STDEXEC_ATTRIBUTE(no_unique_address) _Receiver __rec_;
         __immovable_operation_storage __storage_{};
       };
     };
@@ -958,9 +1007,8 @@ namespace exec {
         __immovable_operation_storage (*__connect_)(void*, __receiver_ref_t);
        private:
         template <sender_to<__receiver_ref_t> _Sender>
-        STDEXEC_MEMFN_DECL(
-          auto
-          __create_vtable)(this __mtype<__vtable>, __mtype<_Sender>) noexcept -> const __vtable* {
+        STDEXEC_MEMFN_DECL(auto __create_vtable)(this __mtype<__vtable>, __mtype<_Sender>) noexcept
+          -> const __vtable* {
           static const __vtable __vtable_{
             {*__create_vtable(__mtype<__query_vtable<_SenderQueries>>{}, __mtype<_Sender>{})},
             [](void* __object_pointer, __receiver_ref_t __receiver)
@@ -983,7 +1031,7 @@ namespace exec {
 
         template <class _Tag, class... _As>
           requires __callable<const __query_vtable<_SenderQueries>&, _Tag, void*, _As...>
-        auto query(_Tag, _As&&... __as) const //
+        auto query(_Tag, _As&&... __as) const
           noexcept(__nothrow_callable<const __query_vtable<_SenderQueries>&, _Tag, void*, _As...>)
             -> __call_result_t<const __query_vtable<_SenderQueries>&, _Tag, void*, _As...> {
           return __vtable_->__queries()(_Tag{}, __sender_, static_cast<_As&&>(__as)...);
@@ -1021,8 +1069,9 @@ namespace exec {
         }
 
         template <receiver_of<_Sigs> _Rcvr>
-        auto connect(_Rcvr __rcvr) && //
-          -> stdexec::__t<__operation<stdexec::__id<_Rcvr>, __with_inplace_stop_token>> {
+        auto connect(_Rcvr __rcvr) && -> stdexec::__t<
+          __operation<stdexec::__id<_Rcvr>, __with_inplace_stop_token>
+        > {
           return {static_cast<__t&&>(*this), static_cast<_Rcvr&&>(__rcvr)};
         }
 
@@ -1035,8 +1084,8 @@ namespace exec {
     class __scheduler {
       static constexpr std::size_t __buffer_size = 4 * sizeof(void*);
       template <class _Ty>
-      static constexpr bool __is_small =
-        sizeof(_Ty) <= __buffer_size && alignof(_Ty) <= alignof(std::max_align_t);
+      static constexpr bool __is_small = sizeof(_Ty) <= __buffer_size
+                                      && alignof(_Ty) <= alignof(std::max_align_t);
 
      public:
       template <class _Scheduler>
@@ -1051,8 +1100,8 @@ namespace exec {
 
       __scheduler(__scheduler&&) noexcept = default;
       __scheduler(const __scheduler&) noexcept = default;
-      __scheduler& operator=(__scheduler&&) noexcept = default;
-      __scheduler& operator=(const __scheduler&) noexcept = default;
+      auto operator=(__scheduler&&) noexcept -> __scheduler& = default;
+      auto operator=(const __scheduler&) noexcept -> __scheduler& = default;
 
       using __sender_t = _ScheduleSender;
 
@@ -1063,12 +1112,11 @@ namespace exec {
 
       template <class _Tag, class... _As>
         requires __callable<const __query_vtable<_SchedulerQueries, false>&, _Tag, void*, _As...>
-      auto query(_Tag, _As&&... __as) const //
-        noexcept(
-          __nothrow_callable<const __query_vtable<_SchedulerQueries, false>&, _Tag, void*, _As...>)
-          -> __call_result_t<const __query_vtable<_SchedulerQueries, false>&, _Tag, void*, _As...> {
-        return __storage_.__get_vtable()->__queries()(
-          _Tag{}, __storage_.__get_object_pointer(), static_cast<_As&&>(__as)...);
+      auto query(_Tag, _As&&... __as) const noexcept(
+        __nothrow_callable<const __query_vtable<_SchedulerQueries, false>&, _Tag, void*, _As...>)
+        -> __call_result_t<const __query_vtable<_SchedulerQueries, false>&, _Tag, void*, _As...> {
+        return __storage_.__get_vtable()
+          ->__queries()(_Tag{}, __storage_.__get_object_pointer(), static_cast<_As&&>(__as)...);
       }
 
      private:
@@ -1083,8 +1131,8 @@ namespace exec {
        private:
         template <scheduler _Scheduler>
         STDEXEC_MEMFN_DECL(
-          auto
-          __create_vtable)(this __mtype<__vtable>, __mtype<_Scheduler>) noexcept -> const __vtable* {
+          auto __create_vtable)(this __mtype<__vtable>, __mtype<_Scheduler>) noexcept
+          -> const __vtable* {
           static const __vtable __vtable_{
             {*__create_vtable(
               __mtype<__query_vtable<_SchedulerQueries, false>>{}, __mtype<_Scheduler>{})},
@@ -1141,36 +1189,30 @@ namespace exec {
     using __t = any_receiver_ref;
     using __id = any_receiver_ref;
 
-    template <
-      stdexec::__none_of<any_receiver_ref, const any_receiver_ref, __env_t, const __env_t> _Receiver>
+    template <stdexec::__none_of<any_receiver_ref, const any_receiver_ref, __env_t, const __env_t>
+                _Receiver>
       requires stdexec::receiver_of<_Receiver, _Completions>
-    any_receiver_ref(_Receiver& __receiver) //
+    any_receiver_ref(_Receiver& __receiver)
       noexcept(stdexec::__nothrow_constructible_from<__receiver_base, _Receiver>)
       : __receiver_(__receiver) {
     }
 
     template <class... _As>
-      requires stdexec::tag_invocable<stdexec::set_value_t, __receiver_base, _As...>
+      requires stdexec::__callable<stdexec::set_value_t, __receiver_base, _As...>
     void set_value(_As&&... __as) noexcept {
-      stdexec::tag_invoke(
-        stdexec::set_value,
-        static_cast<__receiver_base&&>(__receiver_),
-        static_cast<_As&&>(__as)...);
+      stdexec::set_value(static_cast<__receiver_base&&>(__receiver_), static_cast<_As&&>(__as)...);
     }
 
     template <class _Error>
-      requires stdexec::tag_invocable<stdexec::set_error_t, __receiver_base, _Error>
+      requires stdexec::__callable<stdexec::set_error_t, __receiver_base, _Error>
     void set_error(_Error&& __err) noexcept {
-      stdexec::tag_invoke(
-        stdexec::set_error,
-        static_cast<__receiver_base&&>(__receiver_),
-        static_cast<_Error&&>(__err));
+      stdexec::set_error(static_cast<__receiver_base&&>(__receiver_), static_cast<_Error&&>(__err));
     }
 
     void set_stopped() noexcept
-      requires stdexec::tag_invocable<stdexec::set_stopped_t, __receiver_base>
+      requires stdexec::__callable<stdexec::set_stopped_t, __receiver_base>
     {
-      stdexec::tag_invoke(stdexec::set_stopped, static_cast<__receiver_base&&>(__receiver_));
+      stdexec::set_stopped(static_cast<__receiver_base&&>(__receiver_));
     }
 
     auto get_env() const noexcept -> stdexec::env_of_t<__receiver_base> {
@@ -1180,24 +1222,19 @@ namespace exec {
     template <auto... _SenderQueries>
     class any_sender {
       using __sender_base = stdexec::__t<
-        __any::__sender<_Completions, queries<_SenderQueries...>, queries<_ReceiverQueries...>>>;
+        __any::__sender<_Completions, queries<_SenderQueries...>, queries<_ReceiverQueries...>>
+      >;
       __sender_base __sender_;
 
-      template <class _Tag, stdexec::__decays_to<any_sender> Self, class... _As>
-        requires stdexec::tag_invocable<_Tag, stdexec::__copy_cvref_t<Self, __sender_base>, _As...>
-      friend auto tag_invoke(_Tag, Self&& __self, _As&&... __as) //
-        noexcept(
-          stdexec::nothrow_tag_invocable<_Tag, stdexec::__copy_cvref_t<Self, __sender_base>, _As...>) {
-        return stdexec::tag_invoke(
-          _Tag{}, static_cast<Self&&>(__self).__sender_, static_cast<_As&&>(__as)...);
-      }
      public:
       using sender_concept = stdexec::sender_t;
       using completion_signatures = typename __sender_base::completion_signatures;
+      using __t = any_sender;
+      using __id = any_sender;
 
       template <stdexec::__not_decays_to<any_sender> _Sender>
         requires stdexec::sender_to<_Sender, __receiver_base>
-      any_sender(_Sender&& __sender) //
+      any_sender(_Sender&& __sender)
         noexcept(stdexec::__nothrow_constructible_from<__sender_base, _Sender>)
         : __sender_(static_cast<_Sender&&>(__sender)) {
       }
@@ -1207,12 +1244,17 @@ namespace exec {
         return static_cast<__sender_base&&>(__sender_).connect(static_cast<_Receiver&&>(__rcvr));
       }
 
+      auto get_env() const noexcept -> stdexec::env_of_t<__sender_base> {
+        return static_cast<const __sender_base&>(__sender_).get_env();
+      }
+
       template <auto... _SchedulerQueries>
       class any_scheduler {
         // Add the required set_value_t() completions to the schedule-sender.
         using __schedule_completions = stdexec::__concat_completion_signatures<
           _Completions,
-          stdexec::completion_signatures<stdexec::set_value_t()>>;
+          stdexec::completion_signatures<stdexec::set_value_t()>
+        >;
         using __schedule_receiver = any_receiver_ref<__schedule_completions, _ReceiverQueries...>;
 
         template <typename _Tag, typename _Sig>
@@ -1227,8 +1269,10 @@ namespace exec {
 
         using __schedule_sender_queries = stdexec::__minvoke<
           stdexec::__mremove_if<
-            __ret_equals_to<stdexec::get_completion_scheduler_t<stdexec::set_value_t>>>,
-          decltype(_SenderQueries)...>;
+            __ret_equals_to<stdexec::get_completion_scheduler_t<stdexec::set_value_t>>
+          >,
+          decltype(_SenderQueries)...
+        >;
 
 #if STDEXEC_MSVC()
         // MSVCBUG https://developercommunity.visualstudio.com/t/ICE-and-non-ICE-bug-in-NTTP-argument-w/10361081
@@ -1241,7 +1285,8 @@ namespace exec {
 #else
         template <class... _Queries>
         using __schedule_sender_fn = typename __schedule_receiver::template any_sender<
-          stdexec::get_completion_scheduler<stdexec::set_value_t>.template signature<any_scheduler() noexcept>>;
+          stdexec::get_completion_scheduler<stdexec::set_value_t>.template signature<any_scheduler() noexcept>
+        >;
 #endif
         using __schedule_sender =
           stdexec::__mapply<stdexec::__q<__schedule_sender_fn>, __schedule_sender_queries>;
@@ -1252,6 +1297,7 @@ namespace exec {
         __scheduler_base __scheduler_;
 
        public:
+        using scheduler_concept = stdexec::scheduler_t;
         using __t = any_scheduler;
         using __id = any_scheduler;
 
@@ -1266,11 +1312,11 @@ namespace exec {
         }
 
         template <class _Tag, class... _As>
-          requires stdexec::tag_invocable<_Tag, const __scheduler_base&, _As...>
+          requires stdexec::__env::__queryable<const __scheduler_base&, _Tag, _As...>
         auto query(_Tag, _As&&... __as) const
-          noexcept(stdexec::nothrow_tag_invocable<_Tag, const __scheduler_base&, _As...>)
-            -> stdexec::tag_invoke_result_t<_Tag, const __scheduler_base&, _As...> {
-          return stdexec::tag_invoke(_Tag(), __scheduler_, static_cast<_As&&>(__as)...);
+          noexcept(stdexec::__env::__nothrow_queryable<const __scheduler_base&, _Tag, _As...>)
+            -> stdexec::__env::__query_result_t<const __scheduler_base&, _Tag, _As...> {
+          return __scheduler_.query(_Tag{}, static_cast<_As&&>(__as)...);
         }
 
         auto operator==(const any_scheduler&) const noexcept -> bool = default;
