@@ -19,7 +19,6 @@
 #include <cassert>
 #include <compare>
 #include <type_traits>
-#include <utility>
 
 #include "__config.hpp"
 #include "__concepts.hpp"
@@ -65,21 +64,9 @@ namespace stdexec {
     }
   };
 
-  // Because of nvc++ nvbugs#4679848, we can't make __mbool a simple alias for __mconstant,
-  // and because of nvc++ nvbugs#4668709 it can't be a simple alias for std::bool_constant,
-  // either. :-(
-  // template <bool _Bp>
-  // using __mbool = __mconstant<_Bp>;
-
-  template <bool _Bp>
-  struct __mbool : std::bool_constant<_Bp> { };
-
-  using __mtrue = __mbool<true>;
-  using __mfalse = __mbool<false>;
-
   // nvbugs#4679848 and nvbugs#4668709 also preclude __mconstant from representing a compile-time
   // size_t.
-  enum class __muchar : unsigned char {
+  enum class __u8 : unsigned char {
   };
 
 #if STDEXEC_NVCC() || STDEXEC_EDG()
@@ -90,7 +77,7 @@ namespace stdexec {
   using __msize_t = __mconstant<_Np>;
 #else
   template <std::size_t _Np>
-  using __msize_t = __muchar (*)[_Np + 1]; // +1 to avoid zero-size array
+  using __msize_t = __u8 (*)[_Np + 1]; // +1 to avoid zero-size array
 #endif
 
   //! Metafunction selects the first of two type arguments.
@@ -109,7 +96,7 @@ namespace stdexec {
 
   template <class _Tp>
     requires __typename<__mtypeof<_Tp::value>>
-  inline constexpr auto __v<_Tp> = _Tp::value;
+  inline constexpr __mtypeof<_Tp::value> __v<_Tp> = _Tp::value;
 
   // These specializations exist because instantiating a variable template is cheaper than
   // instantiating a class template.
@@ -122,19 +109,18 @@ namespace stdexec {
   template <class _Tp, _Tp _Ip>
   inline constexpr _Tp __v<std::integral_constant<_Tp, _Ip>> = _Ip;
 
+  // `__mtypeof<_Np>` instead of `auto` to work around NVHPC/EDG bug.
   template <auto _Np>
-  inline constexpr auto __v<__mconstant<_Np>> = _Np;
+  inline constexpr __mtypeof<_Np> __v<__mconstant<_Np>> = _Np;
 
   template <std::size_t _Np>
-  inline constexpr std::size_t __v<__muchar (*)[_Np]> = _Np - 1; // see definition of __msize_t
-
-  namespace __pack {
-    template <std::size_t... _Is>
-    struct __t;
-  } // namespace __pack
+  inline constexpr std::size_t __v<__u8 (*)[_Np]> = _Np - 1; // see definition of __msize_t
 
   template <std::size_t... _Is>
-  using __indices = __pack::__t<_Is...> *;
+  struct __iota;
+
+  template <std::size_t... _Is>
+  using __indices = __iota<_Is...> *;
 
 #if STDEXEC_MSVC()
   namespace __pack {
@@ -149,7 +135,7 @@ namespace stdexec {
   } // namespace __pack
 
   template <std::size_t _Np>
-  using __make_indices = //
+  using __make_indices =
     decltype(__pack::__mkidx<__make_integer_seq<__pack::__idx, std::size_t, _Np>>);
 #elif STDEXEC_HAS_BUILTIN(__make_integer_seq)
   namespace __pack {
@@ -187,7 +173,7 @@ namespace stdexec {
   STDEXEC_PRAGMA_PUSH()
   STDEXEC_PRAGMA_IGNORE_MSVC(4293)
 
-  constexpr std::size_t __mpow2(std::size_t __size) noexcept {
+  constexpr auto __mpow2(std::size_t __size) noexcept -> std::size_t {
     --__size;
     __size |= __size >> 1;
     __size |= __size >> 2;
@@ -216,8 +202,7 @@ namespace stdexec {
     }
 #else
     template <std::size_t _Ny>
-    constexpr __mstring(const char (&__str)[_Ny], int = 0) noexcept
-      : __what_{} {
+    constexpr __mstring(const char (&__str)[_Ny], int = 0) noexcept {
       for (auto __i = 0ull; char __ch: __str) {
         __what_[__i++] = __ch;
       }
@@ -255,7 +240,7 @@ namespace stdexec {
       return (_Len < _OtherLen) ? std::strong_ordering::less : std::strong_ordering::greater;
     }
 
-    char __what_[_Len];
+    char __what_[_Len]{};
   };
 
   template <std::size_t _Len>
@@ -298,6 +283,7 @@ namespace stdexec {
 
   template <class _What, class... _With>
   struct _ERROR_ {
+    using __what_t = _What;
     auto operator,(__msuccess) const noexcept -> _ERROR_;
   };
 
@@ -345,9 +331,7 @@ namespace stdexec {
   // nvc++ does not. So we memoize the type computations by
   // indirecting through a class template specialization.
   template <template <class...> class _Fn, class... _Args>
-  using __meval__ =                //
-    typename __i<_Ok<_Args...>>    //
-    ::template __g<_Fn, _Args...>; //
+  using __meval__ = typename __i<_Ok<_Args...>>::template __g<_Fn, _Args...>;
 
   template <template <class...> class _Fn, class... _Args>
   struct __meval_ { };
@@ -362,10 +346,8 @@ namespace stdexec {
   using __meval = __t<__meval_<_Fn, _Args...>>;
 
   template <class _Fn, class... _Args>
-  using __minvoke__ =                     //
-    typename __i<_Ok<_Args...>, _Ok<_Fn>> //
-    ::template __f<_Fn>                   //
-    ::template __f<_Args...>;             //
+  using __minvoke__ =
+    typename __i<_Ok<_Args...>, _Ok<_Fn>>::template __f<_Fn>::template __f<_Args...>;
 
   template <class _Fn, class... _Args>
   struct __minvoke_ { };
@@ -382,18 +364,14 @@ namespace stdexec {
 #else
 
   template <template <class...> class _Fn, class... _Args>
-  using __meval =                  //
-    typename __i<_Ok<_Args...>>    //
-    ::template __g<_Fn, _Args...>; //
+  using __meval = typename __i<_Ok<_Args...>>::template __g<_Fn, _Args...>;
 
   //! Metafunction invocation
   //! Given a metafunction, `_Fn`, and args.
   //! We expect `_Fn::__f` to be type alias template "implementing" the metafunction `_Fn`.
   template <class _Fn, class... _Args>
-  using __minvoke =                       //
-    typename __i<_Ok<_Args...>, _Ok<_Fn>> //
-    ::template __f<_Fn>                   //
-    ::template __f<_Args...>;             //
+  using __minvoke =
+    typename __i<_Ok<_Args...>, _Ok<_Fn>>::template __f<_Fn>::template __f<_Args...>;
 
 #endif
 
@@ -687,7 +665,7 @@ namespace stdexec {
   };
 
   template <std::size_t... _Ns>
-  struct __muncurry_<__pack::__t<_Ns...> *> {
+  struct __muncurry_<__indices<_Ns...>> {
     template <class _Fn>
     using __f = __minvoke<_Fn, __msize_t<_Ns>...>;
   };
@@ -725,7 +703,8 @@ namespace stdexec {
       class... _Cs,
       template <class...> class _Dp = __types,
       class... _Ds,
-      class... _Tail>
+      class... _Tail
+    >
     static auto __f(
       __types<_Ts...> *,
       _Ap<_As...> *,
@@ -733,9 +712,9 @@ namespace stdexec {
       _Cp<_Cs...> * = nullptr,
       _Dp<_Ds...> * = nullptr,
       _Tail *...__tail)
-      -> decltype(__mconcat_<(sizeof...(_Tail) == 0)>::__f(
+      -> __midentity<decltype(__mconcat_<(sizeof...(_Tail) == 0)>::__f(
         static_cast<__types<_Ts..., _As..., _Bs..., _Cs..., _Ds...> *>(nullptr),
-        __tail...));
+        __tail...))>;
   };
 
   template <>
@@ -749,7 +728,8 @@ namespace stdexec {
     template <class... _Args>
     using __f = __mapply<
       _Continuation,
-      decltype(__mconcat_<(sizeof...(_Args) == 0)>::__f({}, static_cast<_Args *>(nullptr)...))>;
+      decltype(__mconcat_<(sizeof...(_Args) == 0)>::__f({}, static_cast<_Args *>(nullptr)...))
+    >;
   };
 
   struct __msize {
@@ -814,17 +794,19 @@ namespace stdexec {
   template <class _Old, class _Continuation = __q<__types>>
   struct __mremove {
     template <class... _Args>
-    using __f = //
-      __minvoke<
-        __mconcat<_Continuation>,
-        __if_c<__same_as<_Args, _Old>, __types<>, __types<_Args>>...>;
+    using __f = __minvoke<
+      __mconcat<_Continuation>,
+      __if_c<__same_as<_Args, _Old>, __types<>, __types<_Args>>...
+    >;
   };
 
   template <class _Pred, class _Continuation = __q<__types>>
   struct __mremove_if {
     template <class... _Args>
-    using __f = //
-      __minvoke<__mconcat<_Continuation>, __if<__minvoke<_Pred, _Args>, __types<>, __types<_Args>>...>;
+    using __f = __minvoke<
+      __mconcat<_Continuation>,
+      __if<__minvoke<_Pred, _Args>, __types<>, __types<_Args>>...
+    >;
   };
 
   template <class _Return>
@@ -941,17 +923,17 @@ namespace stdexec {
         _Fn,
         _Continuation,
         __mapply<__qq<__types>, _List1>,
-        __mapply<__qq<__types>, _List2>> { };
+        __mapply<__qq<__types>, _List2>
+      > { };
 
-  template <             //
-    class _Fn,           //
-    class _Continuation, //
-    template <class...>
-    class _Cp,
-    class... _Cs, //
-    template <class...>
-    class _Dp,
-    class... _Ds>
+  template <
+    class _Fn,
+    class _Continuation,
+    template <class...> class _Cp,
+    class... _Cs,
+    template <class...> class _Dp,
+    class... _Ds
+  >
     requires requires { typename __minvoke<_Continuation, __minvoke<_Fn, _Cs, _Ds>...>; }
   struct __mzip_with2_<_Fn, _Continuation, _Cp<_Cs...>, _Dp<_Ds...>> {
     using __t = __minvoke<_Continuation, __minvoke<_Fn, _Cs, _Ds>...>;
@@ -966,13 +948,14 @@ namespace stdexec {
   template <bool>
   struct __mfind_if_ {
     template <class _Fn, class _Continuation, class _Head, class... _Tail>
-    using __f = //
-      __minvoke<
-        __if_c<
-          __v<__minvoke<_Fn, _Head>>,
-          __mbind_front<_Continuation, _Head>,
-          __mbind_front<__mfind_if_<(sizeof...(_Tail) != 0)>, _Fn, _Continuation>>,
-        _Tail...>;
+    using __f = __minvoke<
+      __if_c<
+        __v<__minvoke<_Fn, _Head>>,
+        __mbind_front<_Continuation, _Head>,
+        __mbind_front<__mfind_if_<(sizeof...(_Tail) != 0)>, _Fn, _Continuation>
+      >,
+      _Tail...
+    >;
   };
 
   template <>
@@ -1042,14 +1025,23 @@ namespace stdexec {
     using __f = __mor<__minvoke<_Fn, _Args>...>;
   };
 
-// C++23 pack indexing is disabled for clang because of
-// https://github.com/llvm/llvm-project/issues/116105
-#if defined(__cpp_pack_indexing) && !STDEXEC_CLANG()
+#if !STDEXEC_STD_NO_PACK_INDEXING()
+  STDEXEC_PRAGMA_PUSH()
+  STDEXEC_PRAGMA_IGNORE_GNU("-Wc++26-extensions")
+
+  template <bool>
+  struct __m_at_ {
+    template <class _Np, class... _Ts>
+    using __f = _Ts...[__v<_Np>];
+  };
+
   template <class _Np, class... _Ts>
-  using __m_at = _Ts...[__v<_Np>];
+  using __m_at = __minvoke<__m_at_<__v<_Np> == ~0ul>, _Np, _Ts...>;
 
   template <std::size_t _Np, class... _Ts>
-  using __m_at_c = _Ts...[_Np];
+  using __m_at_c = __minvoke<__m_at_<_Np == ~0ul>, __msize_t<_Np>, _Ts...>;
+
+  STDEXEC_PRAGMA_POP()
 #elif STDEXEC_HAS_BUILTIN(__type_pack_element)
   template <bool>
   struct __m_at_ {
@@ -1124,19 +1116,26 @@ namespace stdexec {
   using __3 = __placeholder<3>;
 
 #if defined(__cpp_pack_indexing)
+  STDEXEC_PRAGMA_PUSH()
+  STDEXEC_PRAGMA_IGNORE_GNU("-Wc++26-extensions")
+
   template <std::size_t _Np>
   struct __nth_pack_element_t {
     template <class... _Ts>
-    STDEXEC_ATTRIBUTE((always_inline)) constexpr decltype(auto) operator()(_Ts &&...__ts) const noexcept {
+    STDEXEC_ATTRIBUTE(always_inline)
+    constexpr auto operator()(_Ts &&...__ts) const noexcept -> decltype(auto) {
       static_assert(_Np < sizeof...(_Ts));
       return static_cast<_Ts...[_Np] &&>(__ts...[_Np]);
     }
   };
+
+  STDEXEC_PRAGMA_POP()
 #else
   template <class... _Ignore>
   struct __nth_pack_element_impl {
     template <class _Ty, class... _Us>
-    STDEXEC_ATTRIBUTE((always_inline)) constexpr _Ty &&operator()(_Ignore..., _Ty &&__t, _Us &&...) const noexcept {
+    STDEXEC_ATTRIBUTE(always_inline)
+    constexpr _Ty &&operator()(_Ignore..., _Ty &&__t, _Us &&...) const noexcept {
       return static_cast<decltype(__t) &&>(__t);
     }
   };
@@ -1144,12 +1143,14 @@ namespace stdexec {
   template <std::size_t _Np>
   struct __nth_pack_element_t {
     template <std::size_t... _Is>
-    STDEXEC_ATTRIBUTE((always_inline)) static constexpr auto __impl(__indices<_Is...>) noexcept {
+    STDEXEC_ATTRIBUTE(always_inline)
+    static constexpr auto __impl(__indices<_Is...>) noexcept {
       return __nth_pack_element_impl<__ignore_t<_Is>...>();
     }
 
     template <class... _Ts>
-    STDEXEC_ATTRIBUTE((always_inline)) constexpr decltype(auto) operator()(_Ts &&...__ts) const noexcept {
+    STDEXEC_ATTRIBUTE(always_inline)
+    constexpr decltype(auto) operator()(_Ts &&...__ts) const noexcept {
       static_assert(_Np < sizeof...(_Ts));
       return __impl(__make_indices<_Np>())(static_cast<_Ts &&>(__ts)...);
     }
@@ -1162,7 +1163,8 @@ namespace stdexec {
   template <auto... _Vs>
   struct __mliterals {
     template <std::size_t _Np>
-    STDEXEC_ATTRIBUTE((always_inline)) static constexpr auto __nth() noexcept {
+    STDEXEC_ATTRIBUTE(always_inline)
+    static constexpr auto __nth() noexcept {
       return stdexec::__nth_pack_element<_Np>(_Vs...);
     }
   };
@@ -1170,179 +1172,45 @@ namespace stdexec {
   template <std::size_t _Np>
   struct __nth_member {
     template <class _Ty>
-    STDEXEC_ATTRIBUTE((always_inline)) constexpr decltype(auto) operator()(_Ty &&__ty) const noexcept {
+    STDEXEC_ATTRIBUTE(always_inline)
+    constexpr auto operator()(_Ty &&__ty) const noexcept -> decltype(auto) {
       return static_cast<_Ty &&>(__ty).*(__ty.__mbrs_.template __nth<_Np>());
     }
   };
 
-  template <class _Ty, std::size_t _Offset = 0>
-  struct __mdispatch_ {
-    template <class... _Ts>
-    auto operator()(_Ts &&...) const noexcept(noexcept(_Ty{})) -> _Ty {
-      return _Ty{};
-    }
-  };
-
-  template <std::size_t _Np, std::size_t _Offset>
-  struct __mdispatch_<__placeholder<_Np>, _Offset> {
-    template <class... _Ts>
-    auto operator()(_Ts &&...__ts) const noexcept -> decltype(auto) {
-      return stdexec::__nth_pack_element<_Np + _Offset>(static_cast<_Ts &&>(__ts)...);
-    }
-  };
-
-  template <std::size_t _Np, std::size_t _Offset>
-  struct __mdispatch_<__placeholder<_Np> &, _Offset> {
-    template <class... _Ts>
-    auto operator()(_Ts &&...__ts) const noexcept -> decltype(auto) {
-      return stdexec::__nth_pack_element<_Np + _Offset>(__ts...);
-    }
-  };
-
-  template <std::size_t _Np, std::size_t _Offset>
-  struct __mdispatch_<__placeholder<_Np> &&, _Offset> {
-    template <class... _Ts>
-    auto operator()(_Ts &&...__ts) const noexcept -> decltype(auto) {
-      return std::move(stdexec::__nth_pack_element<_Np + _Offset>(__ts...));
-    }
-  };
-
-  template <std::size_t _Np, std::size_t _Offset>
-  struct __mdispatch_<const __placeholder<_Np> &, _Offset> {
-    template <class... _Ts>
-    auto operator()(_Ts &&...__ts) const noexcept -> decltype(auto) {
-      return std::as_const(stdexec::__nth_pack_element<_Np + _Offset>(__ts...));
-    }
-  };
-
-  template <class _Ret, class... _Args, std::size_t _Offset>
-  struct __mdispatch_<_Ret (*)(_Args...), _Offset> {
-    template <class... _Ts>
-      requires(__callable<__mdispatch_<_Args, _Offset>, _Ts...> && ...)
-           && __callable<_Ret, __call_result_t<__mdispatch_<_Args, _Offset>, _Ts...>...>
-    auto operator()(_Ts &&...__ts) const
-      noexcept(__nothrow_callable<_Ret, __call_result_t<__mdispatch_<_Args, _Offset>, _Ts...>...>)
-        -> __call_result_t<_Ret, __call_result_t<__mdispatch_<_Args, _Offset>, _Ts...>...> {
-      return _Ret{}(__mdispatch_<_Args, _Offset>{}(static_cast<_Ts &&>(__ts)...)...);
-    }
-  };
-
-  template <class _Ret, class... _Args, std::size_t _Offset>
-  struct __mdispatch_<_Ret (*)(_Args..., ...), _Offset> {
-    static_assert(_Offset == 0, "nested pack expressions are not supported");
-    using _Pattern = __mback<_Args...>;
-    static constexpr std::size_t __offset =
-      __get_placeholder_offset(static_cast<__mtype<_Pattern> *>(nullptr));
-
-    struct __impl {
-      template <std::size_t... _Idx, class... _Ts>
-        requires(__callable<__mdispatch_<_Args>, _Ts...> && ...)
-               && (__callable<__mdispatch_<_Pattern, _Idx + 1>, _Ts...> && ...)
-               && __callable< //
-                  _Ret,
-                  __call_result_t<__mdispatch_<_Args>, _Ts...>...,
-                  __call_result_t<__mdispatch_<_Pattern, _Idx + 1>, _Ts...>...>
-      auto operator()(__indices<_Idx...>, _Ts &&...__ts) const
-        noexcept(__nothrow_callable<                              //
-                 _Ret,                                            //
-                 __call_result_t<__mdispatch_<_Args>, _Ts...>..., //
-                 __call_result_t<__mdispatch_<_Pattern, _Idx + 1>, _Ts...>...>)
-          -> __call_result_t< //
-            _Ret,
-            __call_result_t<__mdispatch_<_Args>, _Ts...>...,
-            __call_result_t<__mdispatch_<_Pattern, _Idx + 1>, _Ts...>...> {
-        return _Ret()(                                            //
-          __mdispatch_<_Args>()(static_cast<_Ts &&>(__ts)...)..., //
-          __mdispatch_<_Pattern, _Idx + 1>()(static_cast<_Ts &&>(__ts)...)...);
-      }
-    };
-
-    template <class... _Ts>
-      requires(__offset < sizeof...(_Ts))
-           && __callable<__impl, __make_indices<sizeof...(_Ts) - __offset - 1>, _Ts...>
-    auto operator()(_Ts &&...__ts) const
-      noexcept(__nothrow_callable<__impl, __make_indices<sizeof...(_Ts) - __offset - 1>, _Ts...>)
-        -> __msecond<
-          __if_c<(__offset < sizeof...(_Ts))>,
-          __call_result_t<__impl, __make_indices<sizeof...(_Ts) - __offset - 1>, _Ts...>> {
-      return __impl()(__make_indices<sizeof...(_Ts) - __offset - 1>(), static_cast<_Ts &&>(__ts)...);
-    }
-
-    template <class... _Ts>
-      requires(sizeof...(_Ts) == __offset)
-           && __callable<__mdispatch_<__minvoke<__mpop_back<__qf<_Ret>>, _Args...> *>, _Ts...>
-    auto operator()(_Ts &&...__ts) const //
-      noexcept(
-        __nothrow_callable<__mdispatch_<__minvoke<__mpop_back<__qf<_Ret>>, _Args...> *>, _Ts...>)
-        -> __msecond<
-          __if_c<(sizeof...(_Ts) == __offset)>,
-          __call_result_t<__mdispatch_<__minvoke<__mpop_back<__qf<_Ret>>, _Args...> *>, _Ts...>> {
-      return __mdispatch_<__minvoke<__mpop_back<__qf<_Ret>>, _Args...> *>()(
-        static_cast<_Ts &&>(__ts)...);
-    }
-  };
-
-  template <class _Ty>
-  struct __mdispatch { };
-
-  template <class _Ret, class... _Args>
-  struct __mdispatch<_Ret(_Args...)> : __mdispatch_<_Ret (*)(_Args...)> { };
-
-  template <class _Ret, class... _Args>
-  struct __mdispatch<_Ret(_Args..., ...)> : __mdispatch_<_Ret (*)(_Args..., ...)> { };
-
-  template <class _Ty, class... _Ts>
-  concept __dispatchable = __callable<__mdispatch<_Ty>, _Ts...>;
-
-  template <class _Ty, class... _Ts>
-  concept __nothrow_dispatchable = __nothrow_callable<__mdispatch<_Ty>, _Ts...>;
-
-  template <class _Ty, class... _Ts>
-  using __dispatch_result_t = __call_result_t<__mdispatch<_Ty>, _Ts...>;
-
-  template <class _Signature, class... _Args>
-  using __try_dispatch_ = __mbool<__dispatchable<_Signature, _Args...>>;
-
-  template <class _Signatures, class _Continuation = __q<__mfront>>
-  struct __which { };
-
-  template <template <class...> class _Cp, class... _Signatures, class _Continuation>
-  struct __which<_Cp<_Signatures...>, _Continuation> {
-    template <class... _Args>
-    using __f = //
-      __minvoke<__mfind_if<__mbind_back_q<__try_dispatch_, _Args...>, _Continuation>, _Signatures...>;
-  };
-
-  template <class _Signatures, class _DefaultFn, class... _Args>
-  using __make_dispatcher = //
-    __minvoke<__mtry_catch<__mcompose<__q<__mdispatch>, __which<_Signatures>>, _DefaultFn>, _Args...>;
-
   template <class _Set, class... _Ty>
   concept __mset_contains = (STDEXEC_IS_BASE_OF(__mtype<_Ty>, _Set) && ...);
 
+  struct __mset_nil;
+
   namespace __set {
     template <class... _Ts>
-    struct __inherit { };
+    struct __inherit : __mtype<__mset_nil> {
+      template <template <class...> class _Fn>
+      using rebind = _Fn<_Ts...>;
+    };
 
     template <class _Ty, class... _Ts>
     struct __inherit<_Ty, _Ts...>
       : __mtype<_Ty>
-      , __inherit<_Ts...> { };
+      , __inherit<_Ts...> {
+      template <template <class...> class _Fn>
+      using rebind = _Fn<_Ty, _Ts...>;
+    };
 
     template <class... _Set>
     auto operator+(__inherit<_Set...> &) -> __inherit<_Set...>;
 
     template <class... _Set, class _Ty>
-    auto operator%(__inherit<_Set...> &, __mtype<_Ty> &) //
-      -> __if_c<                                         //
-        __mset_contains<__inherit<_Set...>, _Ty>,
-        __inherit<_Set...>,
-        __inherit<_Ty, _Set...>> &;
+    auto operator%(__inherit<_Set...> &, __mtype<_Ty> &) -> __if_c<
+      __mset_contains<__inherit<_Set...>, _Ty>,
+      __inherit<_Set...>,
+      __inherit<_Ty, _Set...>
+    > &;
 
     template <class _ExpectedSet, class... _Ts>
-    concept __mset_eq =                                           //
-      (sizeof...(_Ts) == __v<__mapply<__msize, _ExpectedSet>>) && //
-      __mset_contains<_ExpectedSet, _Ts...>;
+    concept __mset_eq = (sizeof...(_Ts) == __v<__mapply<__msize, _ExpectedSet>>)
+                     && __mset_contains<_ExpectedSet, _Ts...>;
 
     template <class _ExpectedSet>
     struct __eq {
