@@ -323,9 +323,14 @@ struct __memory_impl<__back_end::__thrust_gpu>
 // dispatch framework.
 template <> struct __memory_impl<__back_end::__gpu_multicore> { };
 
-// There are no OpenACC implementations of parallel algorithms in <memory> at
-// the moment.
-template <> struct __memory_impl<__back_end::__openacc> { };
+template <> struct __memory_impl<__back_end::__openacc> {
+
+#if __NVCOMPILER_STDPAR_OPENACC_GPU || _NVHPC_STDPAR_GPU
+
+  // No <memory> algorithms implemented in OpenACC yet.
+
+#endif // __NVCOMPILER_STDPAR_OPENACC_GPU || _NVHPC_STDPAR_GPU
+};
 
 // Each overload of each parallel algorithm has a helper class that packages up
 // the arguments so they can be passed around within the dispatch framework.
