@@ -230,8 +230,9 @@ struct __stdpar_call_impl<_F, _EP, true, _Valid, _First, _Rest...> {
                  ::__call(__f);
     }
 #else
-    static_assert(false, "Internal error: stdpar gpu-multicore back end used "
-                         "when GPU/multicore mode is not enabled.");
+    static_assert(sizeof(_F) < 1,
+                  "Internal error: stdpar gpu-multicore back end used when "
+                  "GPU/multicore mode is not enabled.");
     throw 0;
 #endif
   }
